@@ -1,5 +1,5 @@
 from typing import Generator
-from typing import Iterable
+from typing import Iterable, Mapping
 from typing import Sized
 
 from _pytest._io.saferepr import saferepr
@@ -7,14 +7,21 @@ from _pytest.assertion.util import _compare_eq_any
 
 
 class UnorderedList(list):
-    def __init__(self, expected: Iterable):
+    def __init__(self, expected: Iterable, check_type: bool = True):
         if not isinstance(expected, Iterable):
             raise TypeError(
                 "cannot make unordered comparisons to non-iterable: {!r}".format(expected)
             )
+        if isinstance(expected, Mapping):
+            raise TypeError(
+                "cannot make unordered comparisons to mapping: {!r}".format(expected)
+            )
         super().__init__(expected)
+        self._expected_type = type(expected) if check_type else None
 
     def __eq__(self, actual: Iterable) -> bool:
+        if self._expected_type is not None and self._expected_type != type(actual):
+            return False
         if not isinstance(actual, Iterable):
             return self.copy() == actual
         if not isinstance(actual, Sized):
@@ -24,11 +31,14 @@ class UnorderedList(list):
         extra_left, extra_right = _compare_eq_unordered(self, actual)
         return not extra_left and not extra_right
 
+    def __ne__(self, actual: Iterable) -> bool:
+        return not (self == actual)
+
 
 def unordered(*args) -> UnorderedList:
-    if len(args) == 1 and isinstance(args[0], Generator):
-        return UnorderedList(args[0])
-    return UnorderedList(args)
+    if len(args) == 1:
+        return UnorderedList(args[0], check_type=not isinstance(args[0], Generator))
+    return UnorderedList(args, check_type=False)
 
 
 def _compare_eq_unordered(left: Iterable, right: Iterable):
@@ -48,6 +58,11 @@ def pytest_assertrepr_compare(config, op, left, right):
         left_repr = saferepr(left)
         right_repr = saferepr(right)
         result = ["{} {} {}".format(left_repr, op, right_repr)]
+        left_type = left._expected_type if isinstance(left, UnorderedList) else type(left)
+        right_type = right._expected_type if isinstance(right, UnorderedList) else type(right)
+        if left_type and right_type and left_type != right_type:
+            result.append("Type mismatch:")
+            result.append("{} != {}".format(left_type, right_type))
         extra_left, extra_right = _compare_eq_unordered(left, right)
         if len(extra_left) == 1 and len(extra_right) == 1:
             result.append("One item replaced:")
